module Types.RoomsState
  ( RoomsState,
    initRoomsState,
    HasRoomsState (..),
    roomStateDiffers,
    updateRoomState,
  )
where

import ClassyPrelude
import Types.RoomData (RoomsData)

type RoomsState = MVar RoomsData

initRoomsState :: IO RoomsState
initRoomsState = newMVar []

class HasRoomsState a where
  getRoomsState :: a -> RoomsState

updateRoomState ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m
  ) =>
  RoomsData ->
  m ()
updateRoomState newData = do
  state <- getRoomsState <$> ask
  _ <- swapMVar state newData
  return ()

roomStateDiffers ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m
  ) =>
  RoomsData ->
  m Bool
roomStateDiffers newData = do
  currentData <- ask >>= readMVar . getRoomsState
  return $ not $ eqIgnoreOrdering newData currentData

eqIgnoreOrdering :: (Eq a) => [a] -> [a] -> Bool
eqIgnoreOrdering a b = length a == length b && all (`elem` b) a
